@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import absolute_import, unicode_literals
 
 from kombu import Connection, Exchange, Queue, Producer, Consumer
 from kombu.async import Hub
@@ -11,11 +12,11 @@ queue = Queue('asynt', exchange, 'asynt')
 def send_message(conn):
     producer = Producer(conn)
     producer.publish('hello world', exchange=exchange, routing_key='asynt')
-    print('MESSAGE SENT')
+    print('message sent')
 
 
 def on_message(message):
-    print('RECEIVED: %r' % (message.body, ))
+    print('received: {0!r}'.format(message.body))
     message.ack()
     hub.stop()  # <-- exit after one message
 
