@@ -1,27 +1,19 @@
 # -*- coding: utf-8 -*-
-"""
-kombu.async.hub
-===============
-
-Event loop implementation.
-
-"""
-from __future__ import absolute_import
+"""Event loop implementation."""
+from __future__ import absolute_import, unicode_literals
 
 import errno
-
-from collections import deque
+import itertools
 from contextlib import contextmanager
 from time import sleep
-from types import GeneratorType as generator
+from types import GeneratorType as generator  # noqa
 
-from amqp import promise
-
-from kombu.five import Empty, range
+from kombu.five import Empty, python_2_unicode_compatible, range
 from kombu.log import get_logger
-from kombu.utils import cached_property, fileno
-from kombu.utils.compat import get_errno
-from kombu.utils.eventio import READ, WRITE, ERR, poll
+from kombu.utils.compat import fileno
+from kombu.utils.eventio import ERR, READ, WRITE, poll
+from kombu.utils.objects import cached_property
+from vine import Thenable, promise
 
 from .timer import Timer
 
@@ -30,6 +22,10 @@ logger = get_logger(__name__)
 
 _current_loop = None
 
+W_UNKNOWN_EVENT = """\
+Received unknown event %r for fd %r, please contact support!\
+"""
+
 
 class Stop(BaseException):
     """Stops the event loop."""
@@ -45,21 +41,25 @@ def _dummy_context(*args, **kwargs):
 
 
 def get_event_loop():
+    """Get current event loop object."""
     return _current_loop
 
 
 def set_event_loop(loop):
+    """Set the current event loop object."""
     global _current_loop
     _current_loop = loop
     return loop
 
 
+@python_2_unicode_compatible
 class Hub(object):
     """Event loop object.
 
-    :keyword timer: Specify timer object.
-
+    Arguments:
+        timer (kombu.async.Timer): Specify custom timer instance.
     """
+
     #: Flag set if reading from an fd will not block.
     READ = READ
 
@@ -80,7 +80,7 @@ class Hub(object):
         self.writers = {}
         self.on_tick = set()
         self.on_close = set()
-        self._ready = deque()
+        self._ready = set()
 
         self._running = False
         self._loop = None
@@ -139,19 +139,25 @@ class Hub(object):
                 except (MemoryError, AssertionError):
                     raise
                 except OSError as exc:
-                    if get_errno(exc) == errno.ENOMEM:
+                    if exc.errno == errno.ENOMEM:
                         raise
                     logger.error('Error in timer: %r', exc, exc_info=1)
                 except Exception as exc:
                     logger.error('Error in timer: %r', exc, exc_info=1)
         return min(delay or min_delay, max_delay)
 
+    def _remove_from_loop(self, fd):
+        try:
+            self._unregister(fd)
+        finally:
+            self._discard(fd)
+
     def add(self, fd, callback, flags, args=(), consolidate=False):
         fd = fileno(fd)
         try:
             self.poller.register(fd, flags)
         except ValueError:
-            self._discard(fd)
+            self._remove_from_loop(fd)
             raise
         else:
             dest = self.readers if flags & READ else self.writers
@@ -163,8 +169,7 @@ class Hub(object):
 
     def remove(self, fd):
         fd = fileno(fd)
-        self._unregister(fd)
-        self._discard(fd)
+        self._remove_from_loop(fd)
 
     def run_forever(self):
         self._running = True
@@ -184,9 +189,10 @@ class Hub(object):
             self._loop = None
 
     def call_soon(self, callback, *args):
-        handle = promise(callback, args)
-        self._ready.append(handle)
-        return handle
+        if not isinstance(callback, Thenable):
+            callback = promise(callback, args)
+        self._ready.add(callback)
+        return callback
 
     def call_later(self, delay, callback, *args):
         return self.timer.call_after(delay, callback, args)
@@ -207,8 +213,7 @@ class Hub(object):
         writable = fd in self.writers
         on_write = self.writers.get(fd)
         try:
-            self._unregister(fd)
-            self._discard(fd)
+            self._remove_from_loop(fd)
         finally:
             if writable:
                 cb, args = on_write
@@ -218,8 +223,7 @@ class Hub(object):
         readable = fd in self.readers
         on_read = self.readers.get(fd)
         try:
-            self._unregister(fd)
-            self._discard(fd)
+            self._remove_from_loop(fd)
         finally:
             if readable:
                 cb, args = on_read
@@ -247,6 +251,11 @@ class Hub(object):
         self.writers.pop(fd, None)
         self.consolidate.discard(fd)
 
+    def on_callback_error(self, callback, exc):
+        logger.error(
+            'Callback %r raised exception: %r', callback, exc, exc_info=1,
+        )
+
     def create_loop(self,
                     generator=generator, sleep=sleep, min=min, next=next,
                     Empty=Empty, StopIteration=StopIteration,
@@ -259,27 +268,40 @@ class Hub(object):
         consolidate = self.consolidate
         consolidate_callback = self.consolidate_callback
         on_tick = self.on_tick
-        todo = self._ready
         propagate = self.propagate_errors
+        todo = self._ready
 
         while 1:
             for tick_callback in on_tick:
                 tick_callback()
 
-            while todo:
-                item = todo.popleft()
+            # To avoid infinite loop where one of the callables adds items
+            # to self._ready (via call_soon or otherwise), we take pop only
+            # N items from the ready set.
+            # N represents the current number of items on the set.
+            # That way if a todo adds another one to the ready set,
+            # we will break early and allow execution of readers and writers.
+            current_todos = len(todo)
+            for _ in itertools.repeat(None, current_todos):
+                if not todo:
+                    break
+
+                item = todo.pop()
                 if item:
                     item()
 
             poll_timeout = fire_timers(propagate=propagate) if scheduled else 1
+            #  print('[[[HUB]]]: %s' % (self.repr_active(),))
             if readers or writers:
                 to_consolidate = []
                 try:
                     events = poll(poll_timeout)
+                    #  print('[EVENTS]: %s' % (self.repr_events(events),))
                 except ValueError:  # Issue 882
                     raise StopIteration()
 
                 for fd, event in events or ():
+                    general_error = False
                     if fd in consolidate and \
                             writers.get(fd) is None:
                         to_consolidate.append(fd)
@@ -299,6 +321,12 @@ class Hub(object):
                             self.remove_writer(fd)
                             continue
                     elif event & ERR:
+                        general_error = True
+                    else:
+                        logger.info(W_UNKNOWN_EVENT, event, fd)
+                        general_error = True
+
+                    if general_error:
                         try:
                             cb, cbargs = (readers.get(fd) or
                                           writers.get(fd))
@@ -306,12 +334,14 @@ class Hub(object):
                             pass
 
                     if cb is None:
+                        self.remove(fd)
                         continue
+
                     if isinstance(cb, generator):
                         try:
                             next(cb)
                         except OSError as exc:
-                            if get_errno(exc) != errno.EBADF:
+                            if exc.errno != errno.EBADF:
                                 raise
                             hub_remove(fd)
                         except StopIteration:
@@ -337,7 +367,7 @@ class Hub(object):
 
     def repr_events(self, events):
         from .debug import repr_events
-        return repr_events(self, events)
+        return repr_events(self, events or [])
 
     @cached_property
     def scheduler(self):
