@@ -1,54 +1,58 @@
 # -*- coding: utf-8 -*-
-"""
-kombu.async.timer
-=================
-
-Timer scheduling Python callbacks.
-
-"""
-from __future__ import absolute_import
+"""Timer scheduling Python callbacks."""
+from __future__ import absolute_import, unicode_literals
 
 import heapq
 import sys
 
 from collections import namedtuple
 from datetime import datetime
-from functools import wraps
-from time import time
+from functools import total_ordering
 from weakref import proxy as weakrefproxy
 
-from kombu.five import monotonic
+from vine.utils import wraps
+
+from kombu.five import monotonic, python_2_unicode_compatible
 from kombu.log import get_logger
-from kombu.utils.compat import timedelta_seconds
+from time import time as _time
 
 try:
     from pytz import utc
-except ImportError:
+except ImportError:  # pragma: no cover
     utc = None
 
+__all__ = ['Entry', 'Timer', 'to_timestamp']
+
+logger = get_logger(__name__)
+
 DEFAULT_MAX_INTERVAL = 2
 EPOCH = datetime.utcfromtimestamp(0).replace(tzinfo=utc)
 IS_PYPY = hasattr(sys, 'pypy_version_info')
 
-logger = get_logger(__name__)
-
-__all__ = ['Entry', 'Timer', 'to_timestamp']
-
 scheduled = namedtuple('scheduled', ('eta', 'priority', 'entry'))
 
 
-def to_timestamp(d, default_timezone=utc):
+def to_timestamp(d, default_timezone=utc, time=monotonic):
+    """Convert datetime to timestamp.
+
+    If d' is already a timestamp, then that will be used.
+    """
     if isinstance(d, datetime):
         if d.tzinfo is None:
             d = d.replace(tzinfo=default_timezone)
-        return timedelta_seconds(d - EPOCH)
+        diff = _time() - time()
+        return max((d - EPOCH).total_seconds() - diff, 0)
     return d
 
 
+@total_ordering
+@python_2_unicode_compatible
 class Entry(object):
+    """Schedule Entry."""
+
     if not IS_PYPY:  # pragma: no cover
         __slots__ = (
-            'fun', 'args', 'kwargs', 'tref', 'cancelled',
+            'fun', 'args', 'kwargs', 'tref', 'canceled',
             '_last_run', '__weakref__',
         )
 
@@ -58,14 +62,14 @@ class Entry(object):
         self.kwargs = kwargs or {}
         self.tref = weakrefproxy(self)
         self._last_run = None
-        self.cancelled = False
+        self.canceled = False
 
     def __call__(self):
         return self.fun(*self.args, **self.kwargs)
 
     def cancel(self):
         try:
-            self.tref.cancelled = True
+            self.tref.canceled = True
         except ReferenceError:  # pragma: no cover
             pass
 
@@ -73,26 +77,22 @@ class Entry(object):
         return '<TimerEntry: {0}(*{1!r}, **{2!r})'.format(
             self.fun.__name__, self.args, self.kwargs)
 
-    def __hash__(self):
-        return hash((self.fun, self.args))
+    # must not use hash() to order entries
+    def __lt__(self, other):
+        return id(self) < id(other)
 
-    if sys.version_info[0] == 3:  # pragma: no cover
+    @property
+    def cancelled(self):
+        return self.canceled
 
-        def __lt__(self, other):
-            return hash(self) < hash(other)
-
-        def __gt__(self, other):
-            return hash(self) > hash(other)
-
-        def __eq__(self, other):
-            return hash(self) == hash(other)
-
-        def __ne__(self, other):
-            return not self.__eq__(other)
+    @cancelled.setter
+    def cancelled(self, value):
+        self.canceled = value
 
 
 class Timer(object):
-    """ETA scheduler."""
+    """Async timer implementation."""
+
     Entry = Entry
 
     on_error = None
@@ -126,7 +126,7 @@ class Timer(object):
                     tref._last_run = now
                     return fun(*args, **kwargs)
             finally:
-                if not tref.cancelled:
+                if not tref.canceled:
                     last = tref._last_run
                     next = secs - (now - last) if last else secs
                     self.enter_after(next, tref, priority)
@@ -135,13 +135,13 @@ class Timer(object):
         tref._last_run = None
         return self.enter_after(secs, tref, priority)
 
-    def enter_at(self, entry, eta=None, priority=0, time=time):
+    def enter_at(self, entry, eta=None, priority=0, time=monotonic):
         """Enter function into the scheduler.
 
-        :param entry: Item to enter.
-        :keyword eta: Scheduled time as a :class:`datetime.datetime` object.
-        :keyword priority: Unused.
-
+        Arguments:
+            entry (~kombu.async.timer.Entry): Item to enter.
+            eta (datetime.datetime): Scheduled time.
+            priority (int): Unused.
         """
         if eta is None:
             eta = time()
@@ -154,7 +154,7 @@ class Timer(object):
                 return
         return self._enter(eta, priority, entry)
 
-    def enter_after(self, secs, entry, priority=0, time=time):
+    def enter_after(self, secs, entry, priority=0, time=monotonic):
         return self.enter_at(entry, time() + secs, priority)
 
     def _enter(self, eta, priority, entry, push=heapq.heappush):
@@ -176,11 +176,14 @@ class Timer(object):
     def stop(self):
         pass
 
-    def __iter__(self, min=min, nowfun=time,
+    def __iter__(self, min=min, nowfun=monotonic,
                  pop=heapq.heappop, push=heapq.heappush):
-        """This iterator yields a tuple of ``(entry, wait_seconds)``,
+        """Iterate over schedule.
+
+        This iterator yields a tuple of ``(entry, wait_seconds)``,
         where if entry is :const:`None` the caller should wait
-        for ``wait_seconds`` until it polls the schedule again."""
+        for ``wait_seconds`` until it polls the schedule again.
+        """
         max_interval = self.max_interval
         queue = self._queue
 
@@ -196,7 +199,7 @@ class Timer(object):
 
                     if eventB is eventA:
                         entry = eventA[2]
-                        if not entry.cancelled:
+                        if not entry.canceled:
                             yield None, entry
                         continue
                     else:
