@@ -1,18 +1,13 @@
-"""
-kombu.transport
-===============
-
-Built-in transports.
-
-"""
-from __future__ import absolute_import
+"""Built-in transports."""
+from __future__ import absolute_import, unicode_literals
 
 from kombu.five import string_t
-from kombu.syn import _detect_environment
-from kombu.utils import symbol_by_name
+from kombu.utils.compat import _detect_environment
+from kombu.utils.imports import symbol_by_name
 
 
 def supports_librabbitmq():
+    """Return true if :pypi:`librabbitmq` can be used."""
     if _detect_environment() == 'default':
         try:
             import librabbitmq  # noqa
@@ -22,59 +17,39 @@ def supports_librabbitmq():
             return True
 
 
-def _ghettoq(name, new, alias=None):
-    xxx = new   # stupid enclosing
-
-    def __inner():
-        import warnings
-        _new = callable(xxx) and xxx() or xxx
-        gtransport = 'ghettoq.taproot.{0}'.format(name)
-        ktransport = 'kombu.transport.{0}.Transport'.format(_new)
-        this = alias or name
-        warnings.warn("""
-    Ghettoq does not work with Kombu, but there is now a built-in version
-    of the {0} transport.
-
-    You should replace {1!r} with: {2!r}
-        """.format(name, gtransport, this))
-        return ktransport
-
-    return __inner
-
-
 TRANSPORT_ALIASES = {
     'amqp': 'kombu.transport.pyamqp:Transport',
+    'amqps': 'kombu.transport.pyamqp:SSLTransport',
     'pyamqp': 'kombu.transport.pyamqp:Transport',
     'librabbitmq': 'kombu.transport.librabbitmq:Transport',
     'memory': 'kombu.transport.memory:Transport',
     'redis': 'kombu.transport.redis:Transport',
     'SQS': 'kombu.transport.SQS:Transport',
     'sqs': 'kombu.transport.SQS:Transport',
-    'beanstalk': 'kombu.transport.beanstalk:Transport',
     'mongodb': 'kombu.transport.mongodb:Transport',
-    'couchdb': 'kombu.transport.couchdb:Transport',
     'zookeeper': 'kombu.transport.zookeeper:Transport',
-    'django': 'kombu.transport.django:Transport',
     'sqlalchemy': 'kombu.transport.sqlalchemy:Transport',
     'sqla': 'kombu.transport.sqlalchemy:Transport',
     'SLMQ': 'kombu.transport.SLMQ.Transport',
     'slmq': 'kombu.transport.SLMQ.Transport',
-    'ghettoq.taproot.Redis': _ghettoq('Redis', 'redis', 'redis'),
-    'ghettoq.taproot.Database': _ghettoq('Database', 'django', 'django'),
-    'ghettoq.taproot.MongoDB': _ghettoq('MongoDB', 'mongodb'),
-    'ghettoq.taproot.Beanstalk': _ghettoq('Beanstalk', 'beanstalk'),
-    'ghettoq.taproot.CouchDB': _ghettoq('CouchDB', 'couchdb'),
     'filesystem': 'kombu.transport.filesystem:Transport',
-    'zeromq': 'kombu.transport.zmq:Transport',
-    'zmq': 'kombu.transport.zmq:Transport',
-    'amqplib': 'kombu.transport.amqplib:Transport',
     'qpid': 'kombu.transport.qpid:Transport',
+    'sentinel': 'kombu.transport.redis:SentinelTransport',
+    'consul': 'kombu.transport.consul:Transport',
+    'etcd': 'kombu.transport.etcd:Transport',
 }
 
 _transport_cache = {}
 
 
 def resolve_transport(transport=None):
+    """Get transport by name.
+
+    Arguments:
+        transport (Union[str, type]): This can be either
+            an actual transport class, or the fully qualified
+            path to a transport class, or the alias of a transport.
+    """
     if isinstance(transport, string_t):
         try:
             transport = TRANSPORT_ALIASES[transport]
@@ -103,7 +78,6 @@ def get_transport_cls(transport=None):
 
     If the name does not include `"."` (is not fully qualified),
     the alias table will be consulted.
-
     """
     if transport not in _transport_cache:
         _transport_cache[transport] = resolve_transport(transport)
