@@ -1,33 +1,42 @@
+"""Kombu transport using SQLAlchemy as the message store."""
+from __future__ import absolute_import, unicode_literals
+
 import datetime
 
 from sqlalchemy import (Column, Integer, String, Text, DateTime,
                         Sequence, Boolean, ForeignKey, SmallInteger)
+from sqlalchemy.ext.declarative import declarative_base, declared_attr
 from sqlalchemy.orm import relation
-from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.schema import MetaData
 
+class_registry = {}
 metadata = MetaData()
-ModelBase = declarative_base(metadata=metadata)
+ModelBase = declarative_base(metadata=metadata, class_registry=class_registry)
 
 
-class Queue(ModelBase):
-    __tablename__ = 'kombu_queue'
+class Queue(object):
+    """The queue class."""
+
     __table_args__ = {'sqlite_autoincrement': True, 'mysql_engine': 'InnoDB'}
 
     id = Column(Integer, Sequence('queue_id_sequence'), primary_key=True,
                 autoincrement=True)
     name = Column(String(200), unique=True)
-    messages = relation('Message', backref='queue', lazy='noload')
 
     def __init__(self, name):
         self.name = name
 
     def __str__(self):
-        return '<Queue(%s)>' % (self.name)
+        return '<Queue({self.name})>'.format(self=self)
+
+    @declared_attr
+    def messages(cls):
+        return relation('Message', backref='queue', lazy='noload')
 
 
-class Message(ModelBase):
-    __tablename__ = 'kombu_message'
+class Message(object):
+    """The message class."""
+
     __table_args__ = {'sqlite_autoincrement': True, 'mysql_engine': 'InnoDB'}
 
     id = Column(Integer, Sequence('message_id_sequence'),
@@ -36,8 +45,6 @@ class Message(ModelBase):
     sent_at = Column('timestamp', DateTime, nullable=True, index=True,
                      onupdate=datetime.datetime.now)
     payload = Column(Text, nullable=False)
-    queue_id = Column(Integer, ForeignKey('kombu_queue.id',
-                                          name='FK_kombu_message_queue'))
     version = Column(SmallInteger, nullable=False, default=1)
 
     __mapper_args__ = {'version_id_col': version}
@@ -47,7 +54,14 @@ class Message(ModelBase):
         self.queue = queue
 
     def __str__(self):
-        return '<Message(%s, %s, %s, %s)>' % (self.visible,
-                                              self.sent_at,
-                                              self.payload,
-                                              self.queue_id)
+        return '<Message: {0.sent_at} {0.payload} {0.queue_id}>'.format(self)
+
+    @declared_attr
+    def queue_id(self):
+        return Column(
+            Integer,
+            ForeignKey(
+                '%s.id' % class_registry['Queue'].__tablename__,
+                name='FK_kombu_message_queue'
+            )
+        )
