@@ -1,18 +1,11 @@
-"""
-kombu.utils.debug
-=================
-
-Debugging support.
-
-"""
-from __future__ import absolute_import
+"""Debugging support."""
+from __future__ import absolute_import, unicode_literals
 
 import logging
 
-from collections import Callable
-from functools import wraps
+from vine.utils import wraps
 
-from kombu.five import items
+from kombu.five import items, python_2_unicode_compatible
 from kombu.log import get_logger
 
 __all__ = ['setup_logging', 'Logwrapped']
@@ -20,13 +13,17 @@ __all__ = ['setup_logging', 'Logwrapped']
 
 def setup_logging(loglevel=logging.DEBUG, loggers=['kombu.connection',
                                                    'kombu.channel']):
+    """Setup logging to stdout."""
     for logger in loggers:
         l = get_logger(logger)
         l.addHandler(logging.StreamHandler())
         l.setLevel(loglevel)
 
 
+@python_2_unicode_compatible
 class Logwrapped(object):
+    """Wrap all object methods, to log on call."""
+
     __ignore = ('__enter__', '__exit__')
 
     def __init__(self, instance, logger=None, ident=None):
@@ -37,7 +34,7 @@ class Logwrapped(object):
     def __getattr__(self, key):
         meth = getattr(self.instance, key)
 
-        if not isinstance(meth, Callable) or key in self.__ignore:
+        if not callable(meth) or key in self.__ignore:
             return meth
 
         @wraps(meth)
